@@ -24,13 +24,17 @@ Reaction::UI::Widget::Field::DateTime
 
 =head1 DESCRIPTION
 
-See L<Reaction::UI::Widget::Field>
+See L<Reaction::UI::Widget::Field::Mutable.>
 
 =head1 FRAGMENTS
 
+=head2 widget
+
+Sets C<field_name> to the C<value_string> event id of the viewport.
+
 =head2 field
 
-C<content> contains viewport's C<value_string>
+C<content> contains viewport's C<value_string>.
 
 =head1 AUTHORS
 
