@@ -1,22 +1,22 @@
 package # hide from PAUSE
   RTest::TestDB::Baz;
 
-use base qw/DBIx::Class::Core/;
-use metaclass 'Reaction::Meta::Class';
 use Moose;
+extends 'DBIx::Class::Core';
 
-use MooseX::Types::Moose qw/ArrayRef Int/;
-use Reaction::Types::Core qw/NonEmptySimpleStr/;
+use MooseX::Types::Moose qw/ArrayRef Int Bool/;
+use MooseX::Types::Common::String qw/NonEmptySimpleStr/;
 
 has 'id' => (isa => Int, is => 'ro', required => 1);
 has 'name' => (isa => NonEmptySimpleStr, is => 'rw', required => 1);
+has 'bool_field' => (isa => Bool, is => 'rw', required => 0);
 has 'foo_list' => (
-                   isa => ArrayRef,
-                   is => 'rw',
-                   required => 1,
-                   writer => 'set_foo_list',
-                   reader => 'get_foo_list',
-                  );
+  isa => ArrayRef,
+  is => 'rw',
+  required => 1,
+  writer => 'set_foo_list',
+  reader => 'get_foo_list',
+);
 
 around get_foo_list => sub { [ $_[1]->foo_list->all ] };
 
@@ -27,6 +27,11 @@ __PACKAGE__->table('baz');
 __PACKAGE__->add_columns(
   id => { data_type => 'integer', size => 16, is_auto_increment => 1 },
   name => { data_type => 'varchar', size => 255 },
+  bool_field => {
+      data_type => 'char',
+      size => '1',
+      is_nullable => '1'
+  }
 );
 
 __PACKAGE__->set_primary_key('id');
